# -*- coding: utf-8 -*-
from enigma import *
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import ActionMap, NumberActionMap
from Components.ScrollLabel import ScrollLabel
from Components.GUIComponent import *
from Components.MenuList import MenuList
from Components.Input import Input
from Screens.Console import Console
from Plugins.Plugin import PluginDescriptor
from Screens.ImageWizard import ImageWizard
from Plugins.Plugin import PluginDescriptor

import os

###############################################################################
# CONFIG
###############################################################################
# path to CFroot
multiboot_path = "/media/cf"

# path to the readme-file
multiboot_readme = "/media/cf/readme.txt" 

# path to the multiboot.sh
multiboot_script = "/media/cf/multiboot.sh" 

# path to the multiboot.sh during installation
multiboot_script_install = "/var/tmp/multiboot.sh" 
 
# path to the logfile
multiboot_logfile = "/media/cf/multiboot.log" #

# name of the imageinfofile
multiboot_infofile = ".cf-karte"

# versionstring, please do not change, if you do not release a new version!
multiboot_pluginversion = "V2.0"

###############################################################################
# dont change anything after this line unless you know what you are doing !
###############################################################################
def main(session,**kwargs):
    try:    
     	session.open(Multiboot)
    except:
        print "[MULTIBOOT] Pluginexecution failed"
###############################################################################
def autostart(reason,**kwargs):
    if reason == 0:
        print "[MULTIBOOT] no autostart"
###############################################################################
def Plugins(**kwargs):
    return PluginDescriptor(
        name="Multiboot", 
        description="plugin to execute multiboot commands", 
        where = PluginDescriptor.WHERE_PLUGINMENU,
        fnc = main
        )
###############################################################################
###############################################################################
class Multiboot(Screen):
    skin = """
        <screen position="100,100" size="500,400" title="Multiboot Config" >
            <widget name="menu" position="10,10" size="490,390" scrollbarMode="showOnDemand" />
        </screen>"""
        
    def __init__(self, session, args = 0):
        self.skin = Multiboot.skin
        self.session = session
        Screen.__init__(self, session)
        self.menu = args
        list = []
        list.append((_("info"), "info"))
#        list.append((_("boot 0"), "boot0"))
#        list.append((_("boot 1"), "boot1"))
#        list.append((_("boot 2"), "boot2"))
#        list.append((_("boot 3"), "boot3"))
#        list.append((_("copy 1"), "copy1"))
#        list.append((_("copy 2"), "copy2"))
#        list.append((_("copy 3"), "copy3"))
#        list.append((_("--- Wizzard Options ---"), "nothing"))
        list.append((_("boot   image"), "bootwizzard"))
        list.append((_("copy  image"), "copywizzard"))
        list.append((_("delete image"), "deletewizzard"))
        list.append((_("name  image"), "namewizzard"))
        list.append((_("settings copy"), "settingcopywizzard"))
        list.append((_("settings delete"), "settingdeletewizzard"))
#        list.append((_("--- Info Options ---"), "nothing"))
#        list.append((_("autorun"), "autorun"))
#        list.append((_("log"), "log"))
#        list.append((_("--- Service Options ---"), "nothing"))
        list.append((_("reboot Dreambox"), "reboot"))
        list.append((_("halt Dreambox"), "halt"))
        list.append((_("Restart  enigma2"), "kill"))
#        list.append((_("--- Install Options ---"), "nothing"))
        list.append((_("enable  Multiboot"), "enable"))
        list.append((_("disable Multiboot"), "disable"))
        list.append((_("update  Multiboot"), "update"))
#        list.append((_("--- Help Options ---"), "nothing"))
        list.append((_("help"), "help"))        
#        list.append((_("man"), "man"))        
        list.append((_("about"), "about"))        
        self["menu"] = MenuList(list)
        self["actions"] = ActionMap(["WizardActions", "DirectionActions"],{"ok": self.go,"back": self.close,}, -1)
        
    def go(self):
        returnValue = self["menu"].l.getCurrentSelection()[1]
        if returnValue is not None:
           if returnValue is "bootwizzard":
               ImageBootWizzard(self.session)
           elif returnValue is "copywizzard":
               ImageCopyWizzard(self.session)
           elif returnValue is "deletewizzard":
               ImageDeleteWizzard(self.session)
           elif returnValue is "settingcopywizzard":
               SettingCopyWizzard(self.session)
           elif returnValue is "settingdeletewizzard":
               SettingDeleteWizzard(self.session)
           elif returnValue is "namewizzard":
               NameWizzard(self.session)
           # Info
           elif returnValue is "info":
             self.session.open(Console,_("showing Multiboot Info"),["%s info" % (multiboot_script)])
           elif returnValue is "boot0":
             self.session.open(Console,_("Booting from Flash"),["%s boot 0" % (multiboot_script)])
           elif returnValue is "boot1":
             self.session.open(Console,_("Booting from CF Image 1"),["%s boot 1" % (multiboot_script)])
           elif returnValue is "boot2":
             self.session.open(Console,_("Booting from CF Image 2"),["%s boot 2" % (multiboot_script)])
           elif returnValue is "boot3":
             self.session.open(Console,_("Booting from CF Image 3"),["%s boot 3" % (multiboot_script)])
           elif returnValue is "copy1":
             self.session.open(Console,_("Copying Flash to CF Image 1"),["%s copy 1" % (multiboot_script)])
           elif returnValue is "copy2":
             self.session.open(Console,_("Copying Flash to CF Image 2"),["%s copy 2" % (multiboot_script)])
           elif returnValue is "copy3":
             self.session.open(Console,_("Copying Flash to CF Image 3"),["%s copy 3" % (multiboot_script)])
#           elif returnValue is "autorun":
#             self.session.open(Console,_("showing autorun.bat"),["%s autorun" % (multiboot_script)])
#           elif returnValue is "log":
#             self.session.open(Console,_("showing log"),["%s log" % (multiboot_script)])
           # Service options
           elif returnValue is "reboot":
             self.session.open(Console,_("rebooting Dreambox"),["%s reboot" % (multiboot_script)])
           elif returnValue is "halt":
             self.session.open(Console,_("halting Dreambox"),["%s halt" % (multiboot_script)])
           elif returnValue is "kill":
             self.session.open(Console,_("killing enigma2"),["%s kill" % (multiboot_script)])
           # Install options
           elif returnValue is "enable":
             if os.path.exists(multiboot_script_install) == False:
             	self.session.open(Console,_("enable Multiboot on CF Card - executing at /media/cf"),["%s enable" % (multiboot_script)])
             else:
             	self.session.open(Console,_("enable Multiboot on CF Card - executing at /var/tmp"),["%s enable" % (multiboot_script_install)])
           elif returnValue is "disable":
             if os.path.exists(multiboot_script_install) == False:
             	self.session.open(Console,_("disable Multiboot on CF Card - executing at /media/cf"),["%s disable" % (multiboot_script)])
             else:
	        self.session.open(Console,_("disable Multiboot on CF Card - executing at /var/tmp"),["%s disable" % (multiboot_script_install)])
           elif returnValue is "update":
             if os.path.exists(multiboot_script_install) == False:
             	self.session.open(Console,_("updating Multiboot on CF Card - executing at /media/cf"),["%s update" % (multiboot_script)])
             else:
             	self.session.open(Console,_("updating Multiboot on CF Card - executing at /var/tmp"),["%s update" % (multiboot_script_install)])
           # Help options
           elif returnValue is "help":
             self.session.open(Console,_("short help on Multiboot commands"),["%s" % (multiboot_script)])
           elif returnValue is "man":
             self.session.open(Console,_("display README of Multiboot"),["cat %s" % multiboot_readme])
           elif returnValue is "about":
             self.session.open(MessageBox,_("Multiboot Enigma2 Plugin %s\n\nMultiboot written by thowi\nEnigma2-Plugin writen by 3c5x9\n\nEnjoy Multibooting with your remote control !" % multiboot_pluginversion), MessageBox.TYPE_INFO)

###############################################################################
###############################################################################

class ImageCopyWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForSource()
        
    def askForSource(self):
        self.session.openWithCallback(self.askForTarget,ChoiceBox,_("select SOURCE Image to copy"),self.getListInstalledImages())
        
    def askForTarget(self,source):
        if source is None:
            self.skipCopy("SOURCE is NONE, skipping copy")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.askForTargetName,ChoiceBox,_("select TARGET Image to copy"),self.getListInstalledImages())
            
    def askForTargetName(self,target):
        if target is None:
            self.skipCopy("TARGET is NONE, skipping copy")
        else:
            self.target = target[1]
            self.session.openWithCallback(self.processingCopy,InputBox, title=_("Please enter a name for the new Image in Partition %s") %self.target, text="Image%s" %self.target, maxSize=False, type=Input.TEXT)
            
    def processingCopy(self,targetname):
        if targetname is None:
            self.skipCopy("TARGETNAME is NONE, skipping copy")
        else:
            self.targetname = targetname
            self.session.openWithCallback(self.ImageCopy,MessageBox,_("are you sure to copy?\nSOURCE: %s\nTARGET: %s\nTARGETNAME: %s " % (self.source,self.target, self.targetname)), MessageBox.TYPE_YESNO)

    def ImageCopy(self,answer):
        if answer is None:
            self.skipCopy("answer is None")
        if answer is False:
            self.skipCopy("you were not confirming")
        else:
            title = "copying Image %s to Image %s as %s" %(self.source,self.target,self.targetname)
            cmd = "%s copy %s %s \"%s\" " % (multiboot_script,self.target,self.source,self.targetname)
            self.session.open(Console,_(title),[cmd])
            
    def skipCopy(self,reason):
        self.session.open(MessageBox,_("copy was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getListInstalledImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " % infofile
                images.append((_("Partition:  %i EMPTY") %x,"%i" %x))

        return images
        
###############################################################################
###############################################################################

class ImageBootWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.session.openWithCallback(self.bootPartitionSelected,ChoiceBox,_("select Image to boot from"),self.getListBootableImages())        
    def bootPartitionSelected(self,partition):
        if partition is None:
            self.skipBootselect("no partition to boot from was selected - doing nothing")
        else:
            self.target = partition[1]
            self.session.openWithCallback(self.setBootImage,MessageBox,_("are you sure that you want to boot from \nImage: %s " % (self.target)), MessageBox.TYPE_YESNO)
    def setBootImage(self,answer):
        if answer is None:
            self.skipBootselect("answer is None")
        if answer is False:
            self.skipBootselect("you were not confirming")
        else:
           self.session.open(Console,_("configure booting from Partition %s " % self.target),["%s boot %s" % (multiboot_script,self.target)])

    def skipBootselect(self,reason):
        self.session.open(MessageBox,_("selecting a new partition for booting was canceled, because %s" % reason), MessageBox.TYPE_WARNING)

    def getListBootableImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " %infofile
        
        return images

###############################################################################
###############################################################################

class ImageDeleteWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.session.openWithCallback(self.deletePartitionSelected,ChoiceBox,_("select Image to delete"),self.getListBootableImages())        
    def deletePartitionSelected(self,partition):
        if partition is None:
            self.skipBootselect("no partition was selected for delete")
        else:
            self.target = partition[1]
            self.session.openWithCallback(self.deleteImage,MessageBox,_("are you sure to delete Image ?\nImage: %s " % (self.target)), MessageBox.TYPE_YESNO)
    def deleteImage(self,answer):
        if answer is None:
            self.skipBootselect("answer is None")
        if answer is False:
            self.skipBootselect("you were not confirming")
        else:
            self.session.open(Console,_("deleting Partition %s" % self.target),["%s copy %s %s" % (multiboot_script,self.target,self.target)])

    def skipBootselect(self,reason):
        self.session.open(MessageBox,_("deleting a partition canceled, because %s" % reason), MessageBox.TYPE_WARNING)

    def getListBootableImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " %infofile
        
        return images
        
###############################################################################
###############################################################################

class NameWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForTarget()
        
#    def askForSource(self):
#        self.session.openWithCallback(self.askForTarget,ChoiceBox,_("select SOURCE Image to name"),self.getListInstalledImages())
        
    def askForTarget(self):
            self.session.openWithCallback(self.askForTargetName,ChoiceBox,_("select TARGET Image to name"),self.getListInstalledImages())
            
    def askForTargetName(self,target):
        if target is None:
            self.skipCopy("TARGET is NONE, skipping naming")
        else:
            self.target = target[1]
            self.session.openWithCallback(self.processingCopy,InputBox, title=_("Please enter a name for the Image in Partition %s") %self.target, text="Image%s" %self.target, maxSize=False, type=Input.TEXT)
            
    def processingCopy(self,targetname):
        if targetname is None:
            self.skipCopy("TARGETNAME is NONE, skipping naming")
        else:
            self.targetname = targetname
            self.session.openWithCallback(self.ImageCopy,MessageBox,_("are you sure to name?\nTARGET: %s\nTARGETNAME: %s " % (self.target, self.targetname)), MessageBox.TYPE_YESNO)

    def ImageCopy(self,answer):
        if answer is None:
            self.skipCopy("answer is None")
        if answer is False:
            self.skipCopy("you were not confirming")
        else:
            title = "name Image %s as %s" %(self.target,self.targetname)
            cmd = "%s name %s \"%s\" " % (multiboot_script,self.target,self.targetname)
            self.session.open(Console,_(title),[cmd])
            
    def skipCopy(self,reason):
        self.session.open(MessageBox,_("copy was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getListInstalledImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " % infofile
        return images
        
###############################################################################
###############################################################################

class SettingCopyWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForSource()
        
    def askForSource(self):
        self.session.openWithCallback(self.askForTarget,ChoiceBox,_("select SOURCE Image for setting copy"),self.getListInstalledImages())
        
    def askForTarget(self,source):
        if source is None:
            self.skipCopy("SOURCE is NONE, skipping setting copy")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.askForTargetName,ChoiceBox,_("select TARGET Image for setting copy"),self.getListInstalledImages())
            
    def askForTargetName(self,target):
        if target is None:
            self.skipCopy("TARGET is NONE, skipping setting copy")
        else:
            self.target = target[1]
            self.session.openWithCallback(self.ImageCopy,MessageBox,_("are you sure to copy settings ?\nSOURCE: %s\nTARGET: %s" % (self.source,self.target)), MessageBox.TYPE_YESNO)

    def ImageCopy(self,answer):
        if answer is None:
            self.skipCopy("answer is None")
        if answer is False:
            self.skipCopy("you were not confirming")
        else:
            title = "copying settings from Image %s to Image %s" %(self.source,self.target)
            cmd = "%s setting %s %s" % (multiboot_script,self.target,self.source)
            self.session.open(Console,_(title),[cmd])
            
    def skipCopy(self,reason):
        self.session.open(MessageBox,_("setting copy was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getListInstalledImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " % infofile
        return images
        
###############################################################################
###############################################################################

class SettingDeleteWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForTarget()
        
        
    def askForTarget(self):
            self.session.openWithCallback(self.processingCopy,ChoiceBox,_("select TARGET Image for setting delete"),self.getListInstalledImages())
    def processingCopy(self,target):
        if target is None:
            self.skipCopy("TARGET is NONE, skipping setting delete")
        else:
            self.target = target[1]
            self.session.openWithCallback(self.ImageCopy,MessageBox,_("are you sure to delete settings ?\nTARGET: %s" % (self.target)), MessageBox.TYPE_YESNO)

    def ImageCopy(self,answer):
        if answer is None:
            self.skipCopy("answer is None")
        if answer is False:
            self.skipCopy("you were not confirming")
        else:
            title = "deleting settings from Image %s" %(self.target)
            cmd = "%s setting %s %s" % (multiboot_script,self.target,self.target)
            self.session.open(Console,_(title),[cmd])
            
    def skipCopy(self,reason):
        self.session.open(MessageBox,_("setting delete was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getListInstalledImages(self):
        images = []
        images.append((_("Flash"),"0"))
        for x in range(1,4):
            try:
                infofile = "%s%i/%s" % (multiboot_path,x,multiboot_infofile)
                print "[MULTIBOOT] opening ... %s" %infofile
                fp = open(infofile,"r")
                line =fp.readline()
                fp.close()
                print "[MULTIBOOT] name of partition is %s" % line
                images.append((_(line),"%i" %x))
            except IOError:
                print "[MULTIBOOT] File %s not exists " % infofile
        return images
        
###############################################################################
